/*
 * VariableDouble.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;


/**
 * A number object holding a variable double-precision floating-point value.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.1
 * @see net.lucidviews.util.VariableNumber
 */
public class VariableDouble extends VariableNumber
{
    
	/*
	 * Instance variables
	 */
	
	/** The underlying value. */
	private double value;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * Create a variable number to hold a double value.
	 * <br />
	 * The number object is initialised with the given value.
	 * <br />
	 * <br />
	 * @param value the initial value
	 */
	public VariableDouble( double value )
	{
		this.value = value;
	}
	
	/**
	 * Create a variable number to hold a double value.
	 * <br />
	 * The number object is initialised with the given number.
	 * <br />
	 * <br />
	 * @param value the initial value
	 * @throws NullPointerException if the initial value is <code>null</code>
	 */
	public VariableDouble( Number value )
	{
		this.value = value.doubleValue();
	}
	
	
	/*
	 * Implementation of abstract methods in VariableNumber class
	 */

	protected void setValueInternal( int value )
	{
		this.value = (double)value;
	}
	
	protected void setValueInternal( long value )
	{
		this.value = (double)value;
	}
	
	protected void setValueInternal( float value )
	{
		this.value = (double)value;
	}
	
	protected void setValueInternal( double value )
	{
		this.value = value;
	}
	
	
	/*
	 * Implementation of abstract methods in Number class
	 */
	
    public int intValue()
    {
    	return (int)this.value;
    }
    
    public long longValue()
    {
    	return (long)this.value;
    }
    
    public float floatValue()
    {
    	return (float)this.value;
    }

    public double doubleValue()
    {
    	return this.value;
    }
    
    
    /*
     * Overridden methods of Object base class
     */
    
    public String toString()
    {
    	return String.valueOf( this.value );
    }
    
    public int hashCode()
    {
    	long bits = Double.doubleToLongBits( value );
    	return (int)(bits ^ (bits >>> 32));
    }
    
    public boolean equals( Object obj )
    {
    	if (obj instanceof VariableDouble)
    	{
    		VariableDouble variableLongObj = (VariableDouble)obj;
    		
    		return (Double.doubleToLongBits( this.value ) == Double.doubleToLongBits( variableLongObj.value ));
    	}
    	
    	// not a VariableDouble, so can't be equal
    	return false;
    }
    
}


/*
 * Change History
 *
 * $Log: VariableDouble.java,v $
 * Revision 1.1  2006/10/15 21:20:29  Steve
 * Initial revision.
 *
 */