/*
 * VariableLongTest.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import junit.framework.TestCase;


/**
 * Test harness for the <code>VariableLong</code> class.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.2 $
 * @since 1.1
 */
public class VariableLongTest extends TestCase
{
	
	/*
	 * Static methods
	 */
	
	/**
	 * Runs the test in the command console.
	 * <br />
	 * @param args not used
	 */
	public static void main( String[] args )
	{
		junit.textui.TestRunner.run( VariableLongTest.class );
	}
	
	
	/*
	 * Test cases
	 */
	
	public void testEquals()
	{
		VariableLong o1 = new VariableLong( 348933347165026592l );
		VariableLong o2 = new VariableLong( new Long( 348933347165026592l ) );
		VariableLong o3 = new VariableLong( 248933347165026592l );
		
		// test objects that are equal compare as equal
		assertTrue( o1.equals( o2 ) );
		assertTrue( o2.equals( o1 ) );
		
		// test objects that are not equal don't compare as equal
		assertFalse( o1.equals( o3 ) );
		assertFalse( o3.equals( o1 ) );
		assertFalse( o2.equals( o3 ) );
		assertFalse( o3.equals( o2 ) );
	}
	
	public void testHashCode()
	{
		VariableLong o1 = new VariableLong( 348933347165026592l );
		int o1h = o1.hashCode();
		
		VariableLong o2 = new VariableLong( new Long( 348933347165026592l ) );
		int o2h = o2.hashCode();
		
		VariableLong o3 = new VariableLong( 648933347165026592l );
		int o3h = o3.hashCode();
		
		// test hashCode consistently returns the same value
		assertEquals( o1h, o1.hashCode() );
		assertEquals( o2h, o2.hashCode() );
		assertEquals( o3h, o3.hashCode() );
		
		// test hashCodes are equal for objects that are equal
		assertEquals( o1h, o2h );
	}
	
	public void testToString()
	{
		assertEquals( "348933347165026592", new VariableLong( 348933347165026592l ).toString() );
		assertEquals( "348933347165026592", new VariableLong( new Long( 348933347165026592l ) ).toString() );
		assertEquals( "648933347165026592", new VariableLong( 648933347165026592l ).toString() );
	}
	
	public void testValues()
	{
		VariableLong obj = new VariableLong( 648933347165026592l );
		
		assertEquals( (double)648933347165026592l, obj.doubleValue(), 0.000001 );
		assertEquals( (float)648933347165026592l, obj.floatValue(), 0.000001 );
		assertEquals( (long)648933347165026592l, obj.longValue() );
		assertEquals( (int)648933347165026592l, obj.intValue() );
		assertEquals( (short)648933347165026592l, obj.shortValue() );
		assertEquals( (byte)648933347165026592l, obj.byteValue() );
		
		obj.setValue( (int)-584587394 );
		
		assertEquals( (double)-584587394, obj.doubleValue(), 0.000001 );
		assertEquals( (float)-584587394, obj.floatValue(), 0.000001 );
		assertEquals( (long)-584587394, obj.longValue() );
		assertEquals( (int)-584587394, obj.intValue() );
		assertEquals( (short)-584587394, obj.shortValue() );
		assertEquals( (byte)-584587394, obj.byteValue() );
	}
	
	public void secureValue()
	{
		VariableLong obj = new VariableLong( 648933347165026592l );
		obj.secure();
		
		try
		{
			obj.setValue( 0 );
			fail( "Value has been changed when value was secured." );
		}
		catch (IllegalStateException e)
		{
			// expected behaviour
		}
		catch (Exception e)
		{
			fail( "Unexpected exception: " + e );
		}
	}
	
}


/*
 * Change History
 *
 * $Log: VariableLongTest.java,v $
 * Revision 1.2  2007/12/06 21:02:31  Steve
 * Minor changes to provide compatibility with Java 1.4.
 *
 * Revision 1.1  2006/10/15 21:20:29  Steve
 * Initial revision.
 *
 */